"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _webpack = require("webpack");

var _I18nKeysIdentifer = _interopRequireDefault(require("./I18nKeysIdentifer"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * this plugin for the download i18n files
 * So, if need to downoad i18n in browser this plugin handle that
 *
 *
 * this plugin works
 * how it works?
 *  this plugin hook the requireEnsure method and we write logic for current compiled file download logic
 *
 * hinds:-
 *  requireEnsure method was loader for all js chunk's ,
 *  which means if we use dynamic import then this function will create script tag for that dynamic resource
 *  so every dynamic js chunk will load by this function,
 *  So we update this function with when  dynamic jschunk call the nour code also execute
 *
 * some thing  missing  in this definetion
 *
 */
const pluginName = 'DownloadLogicOfI18n';
const MODULE_TYPE = 'json/i18n';

class I18nDownlodLogic {
  constructor({
    filenameTemplate,
    publicPath,
    localeVarName = 'document.documentElement.lang',
    locales
  }) {
    this.locales = locales;
    this.publicPath = publicPath;
    this.options = {
      filenameTemplate,
      localeVarName
    };
    this.finalChunkIds = new Map();
  }

  captureChunkIds(compilation) {
    this.finalChunkIds.clear();
    compilation.chunks.forEach(chunk => {
      this.finalChunkIds.set(chunk.id, {
        original: chunk.id,
        name: chunk.name || null
      });
    });
  }

  getI18nChunkObject(mainChunk) {
    const obj = {};

    for (const chunk of mainChunk.getAllAsyncChunks()) {
      const finalChunkId = chunk.id;

      if (_I18nKeysIdentifer.default.isChunkHasI18n(chunk)) {
        // Store using the final chunk ID
        obj[finalChunkId] = 1; // Also keep track of which chunks have i18n content

        this.finalChunkIds.set(finalChunkId, {
          original: chunk.id,
          name: chunk.name || null
        });
      }
    }

    return obj;
  }

  addDownloadLogicOfI18nInMainTemplate(mainTemplate) {
    // Extend localVars to create a container for tracking loaded i18n chunks.
    mainTemplate.hooks.localVars.tap(pluginName, (source, mainChunk) => {
      const chunkMap = this.getI18nChunkObject(mainChunk);

      if (!Object.keys(chunkMap).length) {
        return source;
      }

      return _webpack.Template.asString([source, '', '// Object to store loaded I18N chunks', '// Using final chunk IDs from webpack', 'var installedI18nChunks = {', _webpack.Template.indent(mainChunk.ids.map(id => `${JSON.stringify(id)}: 0`).join(',\n')), '};']);
    }); // Extend requireEnsure so that whenever a JS chunk is requested,
    // we also download its associated i18n file (if available).

    mainTemplate.hooks.requireEnsure.tap(pluginName, (source, mainChunk, hash) => {
      const chunkMap = this.getI18nChunkObject(mainChunk);

      if (!Object.keys(chunkMap).length) {
        return source;
      }

      const {
        crossOriginLoading
      } = mainTemplate.outputOptions;
      const {
        localeVarName = 'document.documentElement.lang'
      } = this.options; // Use the provided publicPath or fallback to the one provided by Webpack.

      const publicPath = this.publicPath ? JSON.stringify(this.publicPath) : `${mainTemplate.requireFn}.p`;
      return _webpack.Template.asString([source, `
(function() {
  // ${pluginName} I18N loading with final chunk IDs
  var i18nChunks = ${JSON.stringify(chunkMap)};
  var locale = ${localeVarName};
  
  // Check if the current chunkId has an associated i18n resource
  if (installedI18nChunks[chunkId]) {
    promises.push(installedI18nChunks[chunkId]);
  } else if (installedI18nChunks[chunkId] !== 0 && i18nChunks[chunkId]) {
    promises.push(installedI18nChunks[chunkId] = new Promise(function(resolve, reject) {
      // Retrieve the i18n manifest from the global window.desk_i18n_manifest
      var deskManifest = window.desk_i18n_manifest;
      
      if (!deskManifest || !deskManifest.chunks || !deskManifest.chunks[chunkId]) {
        console.warn("No i18n file available for chunk " + chunkId);
        return resolve();
      }
      
      // Replace [locale] in the basePath with the actual locale
      var basePath = deskManifest.basePath.replace("[locale]", locale);
      
      // Construct the final URL for the i18n file
      var srcPath = basePath + deskManifest.chunks[chunkId];
      var fullsrcPath = ${publicPath} + srcPath;
      
      // Avoid duplicate loading by checking existing script tags
      var existingScriptTags = document.getElementsByTagName("script");
      for (var i = 0; i < existingScriptTags.length; i++) {
        var tag = existingScriptTags[i];
        var dataSrc = tag.getAttribute("data-src") || tag.getAttribute("src");
        if (dataSrc === srcPath || dataSrc === fullsrcPath) {
          return resolve();
        }
      }
      
      // Create a script tag to load the i18n file
      var scriptTag = document.createElement("script");
      scriptTag.onload = function() {
        scriptTag.onerror = scriptTag.onload = null;
        resolve();
      };
      
      scriptTag.onerror = function(event) {
        scriptTag.onerror = scriptTag.onload = null;
        var request = (event && event.target && event.target.src) || fullsrcPath;
        var err = new Error("Loading I18N chunk " + chunkId + " failed.\\n(" + request + ")");
        err.code = "I18N_CHUNK_LOAD_FAILED";
        err.request = request;
        delete installedI18nChunks[chunkId];
        if (scriptTag.parentNode) {
          scriptTag.parentNode.removeChild(scriptTag);
        }
        reject(err);
      };
      
      ${mainTemplate.requireFn}.nc && scriptTag.setAttribute("nonce", ${mainTemplate.requireFn}.nc);
      scriptTag.src = fullsrcPath;
      
      ${crossOriginLoading ? `if (scriptTag.src.indexOf(window.location.origin + '/') !== 0) {
            scriptTag.crossOrigin = ${JSON.stringify(crossOriginLoading)};
           }` : ''}
           
      document.body.appendChild(scriptTag);
    }).then(function() {
      installedI18nChunks[chunkId] = 0;
    }));
  }
})()
`]);
    });
  }
  /**
   * Applies the plugin by tapping into the appropriate Webpack hooks.
   * Now captures final chunk IDs after compilation is complete.
   *
   * @param {Compiler} compiler - The Webpack compiler instance.
   */


  apply(compiler) {
    // First capture final chunk IDs after compilation
    compiler.hooks.afterCompile.tap(pluginName, compilation => {
      this.captureChunkIds(compilation);
    }); // Then inject download logic in thisCompilation

    compiler.hooks.thisCompilation.tap(pluginName, compilation => {
      // console.log(`${pluginName}: Adding download logic with final chunk IDs`);
      const {
        mainTemplate
      } = compilation;
      this.addDownloadLogicOfI18nInMainTemplate(mainTemplate);
    });
  }

}

exports.default = I18nDownlodLogic;