"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createMetaManifest = createMetaManifest;

var _webpackSources = require("webpack-sources");

function createMetaManifest(compilation, plugin, skipEmit = false) {
  const {
    detailedManifest
  } = plugin; // Use the plugin's filename property

  const i18nMetaManifestFileName = plugin.i18nMetaManifestFileName;

  if (!skipEmit) {
    console.log('Generating meta manifest:', i18nMetaManifestFileName);
  } // Initialize a streamlined meta manifest structure


  const finalMetaManifest = {
    modules: {}
  }; // Process each chunk's keys into the modules section

  for (const [chunkId, chunkData] of Object.entries(detailedManifest)) {
    const chunkName = chunkData.chunkName || `chunk_${chunkId}`;
    const keysArray = Array.isArray(chunkData.keysUsed) ? chunkData.keysUsed : Array.from(chunkData.keysUsed); // Only add modules that have i18n keys

    if (keysArray.length > 0) {
      finalMetaManifest.modules[chunkName] = {
        id: chunkId,
        keys: keysArray
      };
    }
  } // Only include new or moved keys in the masterKeys section


  if (plugin.extraKeys && plugin.movedKeys) {
    // Combine extra (new) and moved keys
    const masterKeysSet = new Set([...(Array.from(plugin.extraKeys) || []), ...(Array.from(plugin.movedKeys) || [])]); // Convert back to array for the manifest

    finalMetaManifest.masterKeys = Array.from(masterKeysSet); // Log the keys being added to master chunk

    if (finalMetaManifest.masterKeys.length > 0) {
      console.log(`Extra and moved keys to add to master chunk: ${finalMetaManifest.masterKeys.length}`); // Print each key for clarity

      finalMetaManifest.masterKeys.forEach(key => {
        console.log(`  - ${key}`);
      });
    }
  } // Store the generated manifest for potential further use


  plugin.generatedMetaManifest = finalMetaManifest; // Only emit if not skipped

  if (!skipEmit) {
    const metaManifestSource = new _webpackSources.RawSource(JSON.stringify(finalMetaManifest, null, 2));

    if (!compilation.emitAsset) {
      compilation.assets[i18nMetaManifestFileName] = metaManifestSource;
    } else {
      compilation.emitAsset(i18nMetaManifestFileName, metaManifestSource, {
        desc: 'Streamlined chunk-module-i18nKeys mapping with optimized master keys'
      });
    }
  }

  return finalMetaManifest;
}