"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createRegularManifest = createRegularManifest;

var _webpackSources = require("webpack-sources");

function createRegularManifest(compilation, plugin) {
  const {
    regularManifest,
    // e.g., { chunkId: { localeA: pathA, localeB: pathB, ... }, ... }
    locales,
    // e.g., ['en_US', 'fr_FR', ...]
    i18nManifestFileName,
    // e.g., 'i18n-manifest.json'
    deskI18nManifestFileName = 'desk-i18n-manifest.js'
  } = plugin; // 1. Create the final manifest structure.

  const finalManifest = {
    locales: [...locales],
    chunks: {},
    basePath: 'i18n-chunk/[locale]/'
  };

  for (const chunkId of Object.keys(regularManifest)) {
    const localePaths = regularManifest[chunkId];
    if (!localePaths) continue;
    const entries = Object.entries(localePaths);
    if (!entries.length) continue;
    const [someLocale, fullPath] = entries[0];
    if (!fullPath) continue; // Remove the first two segments from the full path to get the basename.

    const pathSegments = fullPath.split('/');
    const basename = pathSegments.slice(2).join('/');
    finalManifest.chunks[chunkId] = basename;
  } // Convert the manifest object to a prettified JSON string.


  const manifestStr = JSON.stringify(finalManifest, null, 2); // 2. Create the JSON asset (plain JSON, without any wrapping).

  const jsonManifestSource = new _webpackSources.RawSource(manifestStr); // 3. Create the manifest code for window.desk_i18n_manifest.

  const deskI18nManifestCode = `
    (function() {
      window.desk_i18n_manifest = ${manifestStr};
    })();
  `;
  const deskI18nManifestSource = new _webpackSources.RawSource(deskI18nManifestCode); // 4. Emit the plain JSON manifest asset.

  compilation.emitAsset(i18nManifestFileName, jsonManifestSource, {
    desc: 'Emitted plain JSON i18n manifest'
  }); // 5. Emit the desk i18n manifest asset.

  compilation.emitAsset(deskI18nManifestFileName, deskI18nManifestSource, {
    desc: 'Emitted manifest attached to window.desk_i18n_manifest'
  });
}